/*
	zillionClient.m
	Mon Dec  9 16:13:27 CET 2002
 */

#import	"ZillionClient.h"
#import	"ZillionServer.h"
#import	"NSProcessInfo+getopt.h"


#define	CONNECTION_RETRIES	10
#define	SLEEP_INTERVAL		60

int main(int argc, const char **argv)
{
	NSAutoreleasePool	*mainPool = [[NSAutoreleasePool alloc] init];

	NSConnection		*theConnection;
	id					server;
	ZillionClient		*client;
	NSDate				*distantFuture = [NSDate distantFuture];
	NSRunLoop			*currentRunLoop = [NSRunLoop currentRunLoop];
	NSProcessInfo		*proc = [NSProcessInfo processInfo];
	NSDictionary		*options = [proc getOptionsFromSpecification:
			[NSArray arrayWithObjects:@"DOname=s", @"doRespawn", nil]
		defaults:[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithInt:1], @"doRespawn", nil]
		];
	BOOL				doRespawn = [[options objectForKey:@"doRespawn"] intValue];
	NSString			*DOname = [options objectForKey:@"DOname"];
	int					i;

	NSLog(@"Starting client [respawning:%@]...", doRespawn? @"YES": @"NO");

	for (i = 0; i < CONNECTION_RETRIES; i++) {
		if (i) {
			NSLog(@"Sleeping...");
			[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:SLEEP_INTERVAL]];
			NSLog(@"Retrying...");
		}
		server = [[NSConnection rootProxyForConnectionWithRegisteredName:ZILLION_SERVER_NAME
			host:nil] retain];
		if (server) break;
		NSLog(@"Couldn't connect to server...");
	}
	if (!server) {
		NSLog(@"Failed to establish connection after %d retries. Exiting.", CONNECTION_RETRIES);
		[mainPool release];
		return 1;
	}
	[server setProtocolForProxy:@protocol(ZillionServerProtocol)];

	client = [[ZillionClient alloc] initWithRespawnStatus:doRespawn andMasterServer:server];

	// if DOname is set we register with that name to await
	// contacts from outside
	if (DOname)	{
		NSLog(@"Registering proxy client with DO name: %@", DOname);
		theConnection = [NSConnection defaultConnection];
		[theConnection setRootObject:client];
		if ([theConnection registerName:DOname] == NO) {
			NSLog(@"Could not register name with PortName server");
		}
	} else {
		[server registerClient:client];
		NSLog(@"registered client...");
	}

	while (![client clientToGoDown])
	{
		[currentRunLoop acceptInputForMode:NSDefaultRunLoopMode beforeDate:distantFuture];
	}

	NSLog(@"going down on client...");
	[server deregisterClient:client];
	[client release];
	[server release];

	[mainPool release];
	return 0;
}
